@@ -102,13 +102,15 @@ namespace oidn {
       {
         init();
       });
+
+      device->wait();
     }
 
     dirty = false;
     dirtyParam = false;
   }
 
-  void UNetFilter::execute()
+  void UNetFilter::execute(bool sync)
   {
     if (dirty)
       throw Exception(Error::InvalidOperation, "changes to the filter are not committed");
@@ -184,11 +186,14 @@ namespace oidn {
 
       // Copy the output image to the final buffer if filtering in-place
       if (outputTemp)
-        outputCopy(*outputTemp, *output);
+        outputCopy(device, *outputTemp, *output);
 
       // Finished
       progress.finish();
     });
+
+    if (sync)
+      device->wait();
   }
 
   void UNetFilter::computeTileSize()
@@ -251,11 +256,16 @@ namespace oidn {
     H = output->height;
     W = output->width;
 
-    if ((color  && color->format  != Format::Float3) ||
-        (albedo && albedo->format != Format::Float3) ||
-        (normal && normal->format != Format::Float3) ||
-        (output->format != Format::Float3))
-      throw Exception(Error::InvalidOperation, "unsupported image format");
+    if (((color  && color->format  != Format::Float3) ||
+         (albedo && albedo->format != Format::Float3) ||
+         (normal && normal->format != Format::Float3)) &&
+        ((color  && color->format  != Format::Half3) ||
+         (albedo && albedo->format != Format::Half3) ||
+         (normal && normal->format != Format::Half3)))
+      throw Exception(Error::InvalidOperation, "unsupported input image format");
+
+    if (output->format != Format::Float3 && output->format != Format::Half3)
+      throw Exception(Error::InvalidOperation, "unsupported output image format");
 
     if ((color  && (color->width  != W || color->height  != H)) ||
         (albedo && (albedo->width != W || albedo->height != H)) ||
@@ -270,10 +280,11 @@ namespace oidn {
     if (device->isVerbose(2))
     {
       std::cout << "Inputs:";
-      if (color)  std::cout << " " << (directional ? "dir" : (hdr ? "hdr" : "ldr"));
-      if (albedo) std::cout << " " << "alb";
-      if (normal) std::cout << " " << "nrm";
+      if (color)  std::cout << " " << (directional ? "dir" : (hdr ? "hdr" : "ldr")) << ":" << color->format;
+      if (albedo) std::cout << " " << "alb" << ":" << albedo->format;
+      if (normal) std::cout << " " << "nrm" << ":" << normal->format;
       std::cout << std::endl;
+      std::cout << "Output: " << output->format << std::endl;
     }
 
     // Select the weights to use
@@ -463,7 +474,8 @@ namespace oidn {
     // Create the nodes
     const bool snorm = directional || (!color && normal);
 
-    inputReorder = net->addInputReorder(net->newTensor(inputReorderDesc, inputReorderOfs),
+    inputReorder = net->addInputReorder("input",
+                                        net->newTensor(inputReorderDesc, inputReorderOfs),
                                         transferFunc, hdr, snorm);
 
     auto encConv0 = net->addConv("enc_conv0",
@@ -474,28 +486,32 @@ namespace oidn {
                                  encConv0->getDst(),
                                  net->newTensor(encConv1Desc, encConv1Ofs));
 
-    auto pool1 = net->addPool(encConv1->getDst(),
+    auto pool1 = net->addPool("pool1",
+                              encConv1->getDst(),
                               net->newTensor(pool1Desc, pool1Ofs));
 
     auto encConv2 = net->addConv("enc_conv2",
                                  pool1->getDst(),
                                  net->newTensor(encConv2Desc, encConv2Ofs));
 
-    auto pool2 = net->addPool(encConv2->getDst(),
+    auto pool2 = net->addPool("pool2",
+                              encConv2->getDst(),
                               net->newTensor(pool2Desc, pool2Ofs));
 
     auto encConv3 = net->addConv("enc_conv3",
                                  pool2->getDst(),
                                  net->newTensor(encConv3Desc, encConv3Ofs));
 
-    auto pool3 = net->addPool(encConv3->getDst(),
+    auto pool3 = net->addPool("pool3",
+                              encConv3->getDst(),
                               net->newTensor(pool3Desc, pool3Ofs));
 
     auto encConv4 = net->addConv("enc_conv4",
                                  pool3->getDst(),
                                  net->newTensor(encConv4Desc, encConv4Ofs));
 
-    auto pool4 = net->addPool(encConv4->getDst(),
+    auto pool4 = net->addPool("pool4",
+                              encConv4->getDst(),
                               net->newTensor(pool4Desc, pool4Ofs));
 
     auto encConv5a = net->addConv("enc_conv5a",
@@ -506,7 +522,8 @@ namespace oidn {
                                   encConv5a->getDst(),
                                   net->newTensor(encConv5bDesc, encConv5bOfs));
 
-    auto upsample4 = net->addUpsample(encConv5b->getDst(),
+    auto upsample4 = net->addUpsample("upsample4",
+                                      encConv5b->getDst(),
                                       net->newTensor(upsample4Desc, upsample4Ofs));
 
     auto decConv4a = net->addConv("dec_conv4a",
@@ -517,7 +534,8 @@ namespace oidn {
                                   decConv4a->getDst(),
                                   net->newTensor(decConv4bDesc, decConv4bOfs));
 
-    auto upsample3 = net->addUpsample(decConv4b->getDst(),
+    auto upsample3 = net->addUpsample("upsample3",
+                                      decConv4b->getDst(),
                                       net->newTensor(upsample3Desc, upsample3Ofs));
 
     auto decConv3a = net->addConv("dec_conv3a",
@@ -528,7 +546,8 @@ namespace oidn {
                                   decConv3a->getDst(),
                                   net->newTensor(decConv3bDesc, decConv3bOfs));
 
-    auto upsample2 = net->addUpsample(decConv3b->getDst(),
+    auto upsample2 = net->addUpsample("upsample2",
+                                      decConv3b->getDst(),
                                       net->newTensor(upsample2Desc, upsample2Ofs));
 
     auto decConv2a = net->addConv("dec_conv2a",
@@ -539,7 +558,8 @@ namespace oidn {
                                   decConv2a->getDst(),
                                   net->newTensor(decConv2bDesc, decConv2bOfs));
 
-    auto upsample1 = net->addUpsample(decConv2b->getDst(),
+    auto upsample1 = net->addUpsample("upsample1",
+                                      decConv2b->getDst(),
                                       net->newTensor(upsample1Desc, upsample1Ofs));
 
     auto decConv1a = net->addConv("dec_conv1a",
@@ -555,7 +575,8 @@ namespace oidn {
                                  net->newTensor(decConv0Desc, decConv0Ofs),
                                  false);
 
-    outputReorder = net->addOutputReorder(decConv0->getDst(),
+    outputReorder = net->addOutputReorder("output",
+                                          decConv0->getDst(),
                                           transferFunc, hdr, snorm);
 
     // Create the temporary output
